/**
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.arquillian.graphene.page;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.jboss.arquillian.graphene.Graphene;
import org.jboss.arquillian.graphene.spi.location.Scheme;

/**
 * Specifies location of the annotated Page Object, relative to the URL of an Arquillian deployment.
 *
 * This annotation can be used to navigate browser to given URL either by:
 *
 * <ul>
 * <li>{@link InitialPage} annotation</li>
 * <li>{@link Graphene#goTo(Class)} method</li>
 * </ul>
 *
 * By default, it navigates to HTTP location, you can override this by specifying particular location by {@link Scheme}
 * annotation parameter.
 *
 * @see Scheme
 *
 * @author Lukas Fryc
 * @author Stefan Miklosovic
 */
@Retention(RetentionPolicy.RUNTIME)
@Target({ ElementType.TYPE })
public @interface Location {

    String value();

    Class<?> scheme() default Scheme.HTTP.class;
}
