/**
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.arquillian.graphene.ftest.enricher;

import static org.junit.Assert.assertNotNull;

import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.drone.api.annotation.Drone;
import org.jboss.arquillian.graphene.page.Page;
import org.jboss.arquillian.junit.Arquillian;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.openqa.selenium.WebDriver;

/**
 * @author <a href="mailto:jhuska@redhat.com">Juraj Huska</a>
 */
@RunWith(Arquillian.class)
@RunAsClient
public class TestInitializingNestedPageObjects {

    @Page
    private InnerPageObject innerPageObject;

    @Page
    private NestedStaticPageObject nestedStaticPageObject;

    @Drone
    private WebDriver browser;

    @Test
    public void testInitializingInnerPageObjects() {
        assertNotNull("The inner page object should be already initialized!", innerPageObject);
    }

    @Test
    public void testInitializingInnerPageObjectsMethodParam(@Page InnerPageObject innerPageObjectParam) {
        assertNotNull("The inner page object should be already initialized!", innerPageObjectParam);
    }

    @Test
    public void testInitializingNesteStaticClasses() {
        assertNotNull("The nested static page object should be already initialized!", nestedStaticPageObject);
    }

    @Test
    public void testInitializingNesteStaticClassesMethodParam(
        @Page NestedStaticPageObject nestedStaticPageObjectParam) {
        assertNotNull("The nested static page object should be already initialized!", nestedStaticPageObjectParam);
    }

    public class InnerPageObject {

    }

    public static class NestedStaticPageObject {

    }
}
